// 
// Some code for very basic file access from kernel modules.
// File access from kernel modules should only really be used for debugging.
// 
// xorloser - February 2010
// www.xorloser.com
// 

#ifndef _KMOD_FILE_H_
#define _KMOD_FILE_H_

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include <linux/file.h>
#include <asm/uaccess.h>	// for get_fs/set_fs
 
#include "PS3Defines.h"


// read in data from file
inline int kfile_read(const char* filepath, void* data, int size)
{
	mm_segment_t old_fs;
	int result_size;
	struct file* fd = NULL;
	
	result_size = -1;
	old_fs = get_fs();
	set_fs(get_ds());
	
	if(filepath == NULL || data == NULL || size < 0)
	{
		PDEBUG("Error bad paramters\n");
		goto end;
	}
	
	fd = filp_open(filepath, O_RDONLY, 0);
	if( IS_ERR(fd) || fd == NULL )
	{
		PDEBUG("Error opening file %s\n", filepath);
		goto end;
	}
	
	result_size = vfs_read(fd, data, size, &fd->f_pos);
	filp_close(fd, NULL);
	//filp_close(fd, current->files);
	
end:
	set_fs(old_fs);
	return result_size;
}


// write out data to file
inline int kfile_write(const char* filepath, const void* data, int size)
{
	mm_segment_t old_fs;
	int result_size;
	struct file* fd = NULL;
	
	result_size = -1;
	old_fs = get_fs();
	set_fs(get_ds());
	
	if(filepath == NULL || data == NULL || size < 0)
	{
		PDEBUG("Error bad paramters\n");
		goto end;
	}
	
	fd = filp_open(filepath, O_WRONLY|O_CREAT, 0666);
	if( IS_ERR(fd) || fd == NULL )
	{
		PDEBUG("Error opening file %s\n", filepath);
		goto end;
	}
	
	result_size = vfs_write(fd, data, size, &fd->f_pos);
	filp_close(fd, NULL);
	
end:
	set_fs(old_fs);
	return result_size;
}


// get the size of a file
inline int kfile_getsize(const char* filepath)
{
	mm_segment_t old_fs;
	int result_size;
	struct file* fd = NULL;
	
	result_size = -1;
	old_fs = get_fs();
	set_fs(get_ds());
	
	if(filepath == NULL)
	{
		PDEBUG("Error bad paramters\n");
		goto end;
	}
	
	fd = filp_open(filepath, O_RDONLY, 0);
	if( IS_ERR(fd) || fd == NULL )
	{
		PDEBUG("Error opening file %s\n", filepath);
		goto end;
	}
	
	result_size = (int)fd->f_dentry->d_inode->i_size;
	filp_close(fd, NULL);
	
end:
	set_fs(old_fs);
	return result_size;
}


#endif // _KMOD_FILE_H_

