// 
// App to dump the PS3 Bootloader from Linux once it has been exploited.
// This requires the xorhack kernel module to be installed.
// 
// xorloser - January 2010
// www.xorloser.com
// 

#define DUMP_NAME	"BL"
#define DUMP1_ADDR	0x000002401FC00000ULL
#define DUMP2_ADDR	0x000002401FFC0000ULL
#define DUMP_SIZE	0x00040000ULL

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdio.h>
#include <malloc.h>

#include "xorhack.h"
#include "xorhack_sc.h"


// get string depicting version of ps3 firmware
const char* get_ps3_ver_str()
{
	static char ver_str[64];
	u64 version;
	
	xorhack_get_version_info(&version);
	sprintf(ver_str, "%x.%x.%x",
		(u32)((version>>32)&0xFFFF),
		(u32)((version>>16)&0xFFFF),
		(u32)((version>> 0)&0xFFFF));
	return ver_str;
}

// dump memory to a file
int dump_mem_to_file(u64 addr, int size, const char* name)
{
	char filename[256];
	FILE* fd;
	u8* data;
	u64 test;
	
	printf("Dumping %s...\n", name);
	sprintf(filename, "%sDump_v%s.bin", name, get_ps3_ver_str());
	
	fd = fopen(filename, "wb");
	if(fd == NULL)
	{
		printf("Error creating file\n");
		return 0;
	}
	
	data = (u8*)malloc(size);
	xorhack_get(data, addr, size);
	if( fwrite(data, 1, size, fd) != size )
		printf("Error writing to dump\n");
	free(data);
	
	fclose(fd);
	printf("Successfully dumped %s\n\n", name);
	return 1;
}


int main(int argc, char* argv[])
{
	u32 test1[8];
	u32 test2[8];
	
	if( xorhack_init() < 0 )
	{
		printf("Error initing xorhack access\n");
		return 1;
	}
	
	if( !xorhack_is_exploited() )
	{
		printf("Error PS3 isn't exploited\n");
		xorhack_free();
		return 1;
	}
	
	// 
	// perform dump (only works if exploit has been successfully triggered)
	// 
	printf("\nPS3 is version: v%s\n", get_ps3_ver_str());
	xorhack_get(test1, DUMP1_ADDR, sizeof(test1));
	xorhack_get(test2, DUMP2_ADDR, sizeof(test2));
	if(		test1[0] == test1[4] )
	{
		dump_mem_to_file(DUMP1_ADDR, DUMP_SIZE, DUMP_NAME);
	}
	else if(test2[0] == test2[4] )
	{
		dump_mem_to_file(DUMP2_ADDR, DUMP_SIZE, DUMP_NAME);
	}
	else
	{
		printf("Count not find bootloader!\n");
	}
	
	// finish up
	xorhack_free();
	return 0;
}



