// 
// App to read real memory once the PS3 has been exploited.
// This requires the xorhack kernel module to be installed.
// 
// WARNING: Using this incorrectly could crash your PS3 or worse!
// 
// xorloser - March 2010
// www.xorloser.com
// 

#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "xorhack.h"

const char G_TITLE[] =	"ps3peek v2.0  -  xorloser 2010\n";
const char G_USAGE[] =	"Usage:     ps3peek [real address] <options>\n"
						"\n"
						"Options:   -s <size> = total size in bytes (default is 1)\n"
						"           -d <size> = size of data blocks (1/2/4/8) (default is 1)\n"
						"           -b        = print binary data (default is onscreen hex)\n"
						"           -a        = disable printing of addresses before hex values (default is addresses enabled)\n"
						"           -c        = disable printing of ascii characters after hex values (default is ascii enabled)\n"
						"\n"
						"Examples:  ps3peek 0 -s 0x1000000 -d 8 -b > HV_Dump.bin\n"
						"           ps3peek 0x20000044024 -d 4\n"
						"           ps3peek 0x10200 -s 0x10\n"
						"\n"
						"WARNING:   Using this incorrectly could crash your PS3 or worse!\n"
						"\n";


void print_hex_data(u64 address, u64 totalSize, u64 dataSize,
					int printAscii, int printAddress)
{
	u64 i, offset;
	for(offset=0; offset<totalSize; offset+=16)
	{
		int line_size = 16;
		if(line_size > totalSize-offset)
			line_size = totalSize-offset;
		
		// print address before hex values
		if( printAddress )
		{
			if(	(address & 0xFFFFFFFF00000000ull) ||
				((address+totalSize) & 0xFFFFFFFF00000000ull) )
			{
				printf("%016llx:  ", address+offset);
			}
			else
			{
				printf("%08llx:  ", address+offset);
			}
		}
		
		for(i=0; i<line_size; i+=dataSize)
		{
			if(i==8) printf(" ");
			if(		dataSize == 1)	printf("%02X ", xorhack_get8(address+offset+i));
			else if(dataSize == 2)	printf("%04X ", xorhack_get16(address+offset+i));
			else if(dataSize == 4)	printf("%08X ", xorhack_get32(address+offset+i));
			else if(dataSize == 8)	printf("%016llX ", xorhack_get64(address+offset+i));
		}
		for(i=line_size; i<16; i+=dataSize)
		{
			if(i==8) printf(" ");
			if(		dataSize == 1)	printf("   ");
			else if(dataSize == 2)	printf("     ");
			else if(dataSize == 4)	printf("         ");
			else if(dataSize == 8)	printf("                 ");
		}
		
		// print ascii characters after hex values
		if( printAscii )
		{
			printf(" :  ");
			for(i=0; i<line_size; i+=1)
			{
				int data = xorhack_get8(address+offset+i);
				printf("%c", isprint(data)?data:'.');
			}
		}
		
		printf("\n");
	}
}


void print_binary_data(u64 address, u64 totalSize, u64 dataSize)
{
	u64 i, offset;
	for(offset=0; offset<totalSize; offset+=16)
	{
		int line_size = 16;
		if(line_size > totalSize-offset)
			line_size = totalSize-offset;
		
		for(i=0; i<line_size; i+=dataSize)
		{
			if(		dataSize == 1)
			{
				u8 data = xorhack_get8(address+offset+i);
				printf("%c", data);
			}
			else if(dataSize == 2)
			{
				u16 data = xorhack_get16(address+offset+i);
				printf("%c%c", (data>>8)&0xFF, data&0xFF);
			}
			else if(dataSize == 4)
			{
				u32 data = xorhack_get32(address+offset+i);
				printf("%c%c%c%c", (data>>24)&0xFF, (data>>16)&0xFF, (data>>8)&0xFF, data&0xFF);
			}
			else if(dataSize == 8)
			{
				u64 data = xorhack_get64(address+offset+i);
				printf("%c%c%c%c%c%c%c%c",
					(int)((data>>56)&0xFF), (int)((data>>48)&0xFF), (int)((data>>40)&0xFF), (int)((data>>32)&0xFF),
					(int)((data>>24)&0xFF), (int)((data>>16)&0xFF), (int)((data>> 8)&0xFF), (int)((data>> 0)&0xFF));
			}
		}
	}
}



int main(int argc, const char* argv[])
{
	// parse args
	u64 address = 0;
	u64 total_size = 1;
	u64 data_size = 1;
	int i, print_binary = 0, print_ascii = 1, print_address = 1;
	if(argc < 2)
	{
		printf("%s", G_TITLE);
		printf("%s", G_USAGE);
		return 1;
	}
	address = strtoull(argv[1], 0, 0);
	for(i=2; i<argc; i++)
	{
		if(	strcasecmp(argv[i], "-s")==0 && (i+1<argc) )
		{
			total_size = strtoull(argv[i+1], 0, 0);
			i++;
			if(total_size == 0)
			{
				printf("Invalid size %lld.\n", total_size);
				return 1;
			}
		}
		else if( strcasecmp(argv[i], "-d")==0 && (i+1<argc) )
		{
			data_size = strtoull(argv[i+1], 0, 0);
			i++;
			if(	data_size != 1 &&
				data_size != 2 &&
				data_size != 4 &
				data_size != 8 )
			{
				printf("Invalid data size %lld. Data size must be 1, 2, 4, or 8 bytes.\n", data_size);
				return 1;
			}
		}
		else if( strcasecmp(argv[i], "-b")==0 )
		{
			print_binary = 1;
		}
		else if( strcasecmp(argv[i], "-a")==0 )
		{
			print_address = 0;
		}
		else if( strcasecmp(argv[i], "-c")==0 )
		{
			print_ascii = 0;
		}
		else
		{
			printf("Invalid or incorrectly used arg \"%s\"\n\n", argv[i]);
			printf("%s", G_TITLE);
			printf("%s", G_USAGE);
			return 1;
		}
	}
	// ensure total size is a multiple of data size
	total_size = (total_size + data_size-1) & ~(data_size-1);
	
	if( xorhack_init() < 0 )
	{
		printf("Error initing xorhack access\n");
		return 1;
	}
	
	if( !xorhack_is_exploited() )
	{
		printf("Error PS3 isn't exploited\n");
		xorhack_free();
		return 1;
	}
	
	if( print_binary )
		print_binary_data(address, total_size, data_size);
	else
		print_hex_data(address, total_size, data_size, print_ascii, print_address);
	
	// finish up
	xorhack_free();
	return 0;
}

