// 
// App to write real memory once the PS3 has been exploited.
// This requires the xorhack kernel module to be installed.
// 
// WARNING: Using this incorrectly could crash your PS3 or worse!
// 
// xorloser - March 2010
// www.xorloser.com
// 

#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "xorhack.h"

const char G_TITLE[] =	"ps3poke v2.0  -  xorloser 2010\n";
const char G_USAGE[] =	"Usage:     ps3poke [real address] <options>\n"
						"\n"
						"Options:   -v <values>   = data values to write to memory\n"
						"           -f <filename> = file containing the data values to write to memory\n"
						"           -s <size>     = total size in bytes, will pad with zeros if bigger than the data provided\n"
						"           -d <size>     = size of data blocks (1/2/4/8) (default is 1)\n"
						"\n"
						"Examples:  ps3poke 0 -s 0x1000000\n"
						"           ps3poke 0x10200 -v \"0x78 0x6f 0x72 0x6c 0x6f 0x73 0x65 0x72 0x20 0x77 0x61 0x73 0x20 0x65 0x72 0x65\" \n"
						"\n"
						"WARNING:   Using this incorrectly could crash your PS3 or worse!\n"
						"\n";


void set_data(u64 address, u8* data, u64 totalSize, u64 dataSize)
{
	u64 offset;
	for(offset=0; offset<totalSize; offset+=dataSize)
	{
		if(dataSize == 1)		xorhack_set8( address+offset, *(u8 *)(data+offset));
		else if(dataSize == 2)	xorhack_set16(address+offset, *(u16*)(data+offset));
		else if(dataSize == 4)	xorhack_set32(address+offset, *(u32*)(data+offset));
		else if(dataSize == 8)	xorhack_set64(address+offset, *(u64*)(data+offset));
	}
}



int main(int argc, const char* argv[])
{
	// parse args
	u64 address = 0;
	u64 total_size = 0;
	u64 data_size = 1;
	int i;
	const char* filename = NULL;
	const char* values_str = NULL;
	u8* buff = NULL;
	
	if(argc < 3)
	{
		printf("%s", G_TITLE);
		printf("%s", G_USAGE);
		return 1;
	}
	address = strtoull(argv[1], 0, 0);
	for(i=2; i<argc; i++)
	{
		if( strcasecmp(argv[i], "-s")==0 && (i+1<argc) )
		{
			total_size = strtoull(argv[i+1], 0, 0);
			i++;
			if(total_size == 0)
			{
				printf("Invalid size %lld.\n", total_size);
				return 1;
			}
		}
		else if( strcasecmp(argv[i], "-d")==0 && (i+1<argc) )
		{
			data_size = strtoull(argv[i+1], 0, 0);
			i++;
			if(	data_size != 1 &&
				data_size != 2 &&
				data_size != 4 &
				data_size != 8 )
			{
				printf("Invalid data size %lld. Data size must be 1, 2, 4, or 8 bytes.\n", data_size);
				return 1;
			}
		}
		else if( strcasecmp(argv[i], "-v")==0 && (i+1<argc) )
		{
			values_str = argv[i+1];
			i++;
		}
		else if( strcasecmp(argv[i], "-f")==0 && (i+1<argc) )
		{
			filename = argv[i+1];
			i++;
		}
		else
		{
			printf("Invalid or incorrectly used arg \"%s\"\n\n", argv[i]);
			printf("%s", G_TITLE);
			printf("%s", G_USAGE);
			return 1;
		}
	}
	if(filename && values_str)
	{
		printf("You cannot specify both values and a file containing values.\n\n");
		printf("%s", G_TITLE);
		printf("%s", G_USAGE);
		return 1;
	}
	
	// get values from file
	if(filename)
	{
		FILE* fd = fopen(filename, "rb");
		if(fd == NULL)
		{
			printf("Error opening %s\n", filename);
			return 1;
		}
		if(total_size == 0)
		{
			fseek(fd, 0, SEEK_END);
			total_size = ftell(fd);
			fseek(fd, 0, SEEK_SET);
		}
		buff = (u8*)malloc(total_size);
		memset(buff, 0, total_size);
		fread(buff, 1, total_size, fd);
		fclose(fd);
	}
	
	// get values from string
	if(values_str)
	{
		const char* str_ptr;
		const char* str_end;
		u8* buff_ptr;
		u8* buff_end;
		int max_size, num_str_len;
		
		if(total_size == 0)
		{
			// assume 1 char per value and 1 char space between values
			max_size = ((strlen(values_str) / 2)+1) * data_size;
		}
		else
		{
			// round maxsize up to a multiple of data size
			max_size = (total_size + data_size-1) & ~(data_size-1);
		}
		buff = (u8*)malloc(max_size);
		memset(buff, 0, max_size);
		buff_ptr = buff;
		buff_end = buff + max_size;
		
		// read in values
		str_end = values_str + strlen(values_str);
		for(str_ptr=values_str; str_ptr<str_end; )
		{
			if(data_size == 1)
			{
				u8 data = (u8)strtoull(str_ptr, 0, 0);
				memcpy(buff_ptr, &data, sizeof(data));
				buff_ptr += sizeof(data);
			}
			else if(data_size == 2)
			{
				u16 data = (u16)strtoull(str_ptr, 0, 0);
				memcpy(buff_ptr, &data, sizeof(data));
				buff_ptr += sizeof(data);
			}
			else if(data_size == 4)
			{
				u32 data = (u32)strtoull(str_ptr, 0, 0);
				memcpy(buff_ptr, &data, sizeof(data));
				buff_ptr += sizeof(data);
			}
			else if(data_size == 8)
			{
				u64 data = (u64)strtoull(str_ptr, 0, 0);
				memcpy(buff_ptr, &data, sizeof(data));
				buff_ptr += sizeof(data);
			}
			
			// update string pointer
			// skip number string and then the string whitespace
			for(num_str_len=0; str_ptr[num_str_len]!=0; num_str_len++)
			{
				if(	str_ptr[num_str_len] == ' ' ||
					str_ptr[num_str_len] == '\t' )
				{
					break;
				}
			}
			for( ; str_ptr[num_str_len]!=0; num_str_len++)
			{
				if(	str_ptr[num_str_len] != ' ' &&
					str_ptr[num_str_len] != '\t' )
				{
					break;
				}
			}
			str_ptr+=num_str_len;
			
			// check buffer pointer is still ok
			if(buff_ptr >= buff_end)
				break;
		}
		
		// finished parsing values string, so work out the total size
		total_size = buff_ptr - buff;
	}
	
	
	if( xorhack_init() < 0 )
	{
		printf("Error initing xorhack access\n");
		free(buff);
		return 1;
	}
	
	if( !xorhack_is_exploited() )
	{
		printf("Error PS3 isn't exploited\n");
		free(buff);
		xorhack_free();
		return 1;
	}
	
	set_data(address, buff, total_size, data_size);
	
	// finish up
	free(buff);
	xorhack_free();
	return 0;
}

