// 
// interface to kernel module to allow easier hacking of the PS3 by allowing 
// lv1 syscalls and exploit triggering from userspare among other things
// 
// xorloser - January 2010
// www.xorloser.com
// 

#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "xorhack.h"
#include "../common/xorhack_ioctl.h"


// handle to device
static int gs_xorhack_fd = -1;


// init the xorhack device
// 
// returns:	0 if completed successfully
//			< 0 if error
int xorhack_init()
{
	// check if already inited
	if( gs_xorhack_fd >= 0 )
		return 0;
	
	gs_xorhack_fd = open("/dev/" XORHACK_NAME, O_RDWR);
	return (gs_xorhack_fd >= 0) ? 0 : -1;
}

// free the xorhack device
// 
// returns:	0 if completed successfully
//			< 0 if error
int xorhack_free()
{
	close(gs_xorhack_fd);
	gs_xorhack_fd = -1;
	return 0;
}


// test if the ps3 has been exploited
// 
// returns:	1 if ps3 has been exploited
//			0 if ps3 hasnt been exploited
int xorhack_is_exploited()
{
	return (xorhack_get64(0x2401FC00000ull) != 0xFFFFFFEC)?1:0;
}


// perform the ps3 exploit
// 
// args:	number of times to loop waiting for the exploit
// returns:	0 if completed successfully
//			< 0 if error
int xorhack_ps3_exploit(int num)
{
	return ioctl(gs_xorhack_fd, IOCTL_PS3_EXPLOIT, num);
}


// syscall that allows the maximum amounts of inputs and outputs
// just specify 0 for all unused input and output values
// 
// args:	syscall number, usually 0 - 255
//			input args
//			output resuts, ret1 is the return value of the syscall itself (r3)
// returns:	0 if completed successfully
//			< 0 if error
int xorhack_syscall(u64 scNum,
	u64  arg1, u64  arg2, u64  arg3, u64  arg4, u64  arg5, u64  arg6, u64  arg7, u64  arg8,
	u64* ret1, u64* ret2, u64* ret3, u64* ret4, u64* ret5, u64* ret6, u64* ret7, u64* ret8)
{
	int ret;
	XorSCParam_t sc_param;
	
	sc_param.scNum = scNum;
	sc_param.args[0] = arg1;
	sc_param.args[1] = arg2;
	sc_param.args[2] = arg3;
	sc_param.args[3] = arg4;
	sc_param.args[4] = arg5;
	sc_param.args[5] = arg6;
	sc_param.args[6] = arg7;
	sc_param.args[7] = arg8;
	
	ret = ioctl(gs_xorhack_fd, IOCTL_LV1_SYSCALL, &sc_param);
	
	if(ret1) *ret1 = sc_param.rets[0];
	if(ret2) *ret2 = sc_param.rets[1];
	if(ret3) *ret3 = sc_param.rets[2];
	if(ret4) *ret4 = sc_param.rets[3];
	if(ret5) *ret5 = sc_param.rets[4];
	if(ret6) *ret6 = sc_param.rets[5];
	if(ret7) *ret7 = sc_param.rets[6];
	if(ret8) *ret8 = sc_param.rets[7];
	
	return ret;
}


// execute code at a given real address
// (the code must be accessable and flagged as executable!)
// 
// args:	real address of code to execute
//			args to send in regs to code
//			return results in regs after running code
void xorhack_exec(u64 realAddr,
	u64 arg1, u64 arg2, u64 arg3, u64 arg4, u64 arg5, u64 arg6, u64 arg7,
	u64* ret1, u64* ret2, u64* ret3, u64* ret4, u64* ret5, u64* ret6, u64* ret7, u64* ret8)
{
	int ret, i;
	XorExecParam_t exec_param;
	memset(&exec_param, 0, sizeof(XorExecParam_t));
	
	exec_param.funcAddr = realAddr;
	exec_param.args[0] = arg1;
	exec_param.args[1] = arg2;
	exec_param.args[2] = arg3;
	exec_param.args[3] = arg4;
	exec_param.args[4] = arg5;
	exec_param.args[5] = arg6;
	exec_param.args[6] = arg7;
	
	ret = ioctl(gs_xorhack_fd, IOCTL_LV1_HVEXEC, &exec_param);
	
	if(ret1) *ret1 = exec_param.rets[0];
	if(ret2) *ret2 = exec_param.rets[1];
	if(ret3) *ret3 = exec_param.rets[2];
	if(ret4) *ret4 = exec_param.rets[3];
	if(ret5) *ret5 = exec_param.rets[4];
	if(ret6) *ret6 = exec_param.rets[5];
	if(ret7) *ret7 = exec_param.rets[6];
	if(ret8) *ret8 = exec_param.rets[7];
}


// convert lpar address to real address
// 
// args:	lpar address
// returns:	real address
u64 xorhack_la2ra(u64 lparAddr)
{
	XorLA2RAParam_t param;
	param.lparAddr = lparAddr;
	ioctl(gs_xorhack_fd, IOCTL_LV1_LA2RA, &param);
	return param.realAddr;
}


// remap memory specifying flags to use
// 
// args:	original lpar address to remap
//			size to remap
//			flags to remap with
// returns:	real address
u64 xorhack_ioremap(u64 addr, u64 size, u64 flags)
{
	XorIoremapParam_t param;
	param.addr = addr;
	param.size = size;
	param.flags= flags;
	ioctl(gs_xorhack_fd, IOCTL_LV1_IOREMAP, &param);
	return param.remappedAddr;
}


// Read data from address into buffer.
// This is used like memcpy.
// (address is in the Real Address space)
// 
// 
// args:	destination address to write to
//			real address to read from
//			size of data to get in bytes
void xorhack_get(void* dest, u64 srcAddress, int size)
{
	int curr_off, curr_size;
	u8* p_dest = (u8*)dest;
	u64 data;
	
	while( size )
	{
		curr_off = (srcAddress & 7);
		curr_size = 8 - (srcAddress & 7);
		if(curr_size > size)
			curr_size = size;
		
		data = xorhack_get64(srcAddress & ~7);
		memcpy(p_dest, ((u8*)&data)+curr_off, curr_size);
		
		p_dest += curr_size;
		srcAddress += curr_size;
		size -= curr_size;
	}
}


// Write data from buffer into address.
// This is used like memcpy.
// (address is in the Real Address space)
// 
// args:	real address to write to
//			source address to read from
//			size of data to set in bytes
void xorhack_set(u64 destAddress, const void* src, int size)
{
	int curr_off, curr_size;
	u8* p_src = (u8*)src;
	u64 data;
	
	while( size )
	{
		curr_off = (destAddress & 7);
		curr_size = 8 - (destAddress & 7);
		if(curr_size > size)
			curr_size = size;
		
		data = xorhack_get64(destAddress & ~7);
		memcpy(((u8*)&data)+curr_off, p_src, curr_size);
		xorhack_set64(destAddress & ~7, data);
		
		p_src += curr_size;
		destAddress += curr_size;
		size -= curr_size;
	}
}


// Writes a data value to one or more consecutive addresses
// This is used like memset.
// (address is in the Real Address space)
// 
// args:	real address to write to
//			source address to read from
//			size of data to set in bytes
void xorhack_fill(u64 destAddress, u8 data, int size)
{
	int i;
	for(i=0; i<size; i++)
	{
		xorhack_set8(destAddress+i, data);
	}
}




// get/set that requires aligned addresses and performs
// the reads/writes in a single access

static inline u8  user_get8(u64 addr)
{
	u64 value;
	xorhack_syscall(35,
		addr, 0,0,0,0,0,0,0,
		&value, 0,0,0,0,0,0,0);
	return value;
}

static inline u16 user_get16(u64 addr)
{
	u64 value;
	xorhack_syscall(34,
		addr, 0,0,0,0,0,0,0,
		&value, 0,0,0,0,0,0,0);
	return value;
}

static inline u32 user_get32(u64 addr)
{
	u64 value;
	xorhack_syscall(33,
		addr, 0,0,0,0,0,0,0,
		&value, 0,0,0,0,0,0,0);
	return value;
}

static inline u64 user_get64(u64 addr)
{
	u64 value;
	xorhack_syscall(32,
		addr, 0,0,0,0,0,0,0,
		&value, 0,0,0,0,0,0,0);
	return value;
}


static inline void user_set8(u64 addr, u8 data)
{
	u64 sc_ret;
	xorhack_syscall(39,
		addr, data,0,0,0,0,0,0,
		&sc_ret, 0,0,0,0,0,0,0);
}

static inline void user_set16(u64 addr, u16 data)
{
	u64 sc_ret;
	xorhack_syscall(38,
		addr, data,0,0,0,0,0,0,
		&sc_ret, 0,0,0,0,0,0,0);
}

static inline void user_set32(u64 addr, u32 data)
{
	u64 sc_ret;
	xorhack_syscall(37,
		addr, data,0,0,0,0,0,0,
		&sc_ret, 0,0,0,0,0,0,0);
}

static inline void user_set64(u64 addr, u64 data)
{
	u64 sc_ret;
	xorhack_syscall(36,
		addr, data,0,0,0,0,0,0,
		&sc_ret, 0,0,0,0,0,0,0);
}





// get/set functions allow unaligned addresses
// If address is unaligned it may internally use multiple accesses.
// If address is aligned it will be done internally in one access.

u8  xorhack_get8(u64 addr)
{
	return user_get8(addr);
}

u16 xorhack_get16(u64 addr)
{
	u64 mask = sizeof(u16)-1;
	u64 addr_overflow = addr & mask;
	u64 addr_masked = addr & ~mask;
	
	if(addr_overflow)
	{
		u64 data = user_get32(addr_masked);
		return data >> (8*(sizeof(u16)-addr_overflow));
	}
	else
	{
		return user_get16(addr);
	}
}

u32 xorhack_get32(u64 addr)
{
	u64 mask = sizeof(u32)-1;
	u64 addr_overflow = addr & mask;
	u64 addr_masked = addr & ~mask;
	
	if(addr_overflow)
	{
		u64 data = user_get64(addr_masked);
		return data >> (8*(sizeof(u32)-addr_overflow));
	}
	else
	{
		return user_get32(addr);
	}
}

u64 xorhack_get64(u64 addr)
{
	u64 mask = sizeof(u64)-1;
	u64 addr_overflow = addr & mask;
	u64 addr_masked = addr & ~mask;
	
	if(addr_overflow)
	{
		u64 data_lo = user_get64(addr_masked);
		u64 data_hi = user_get64(addr_masked + 8);
		return (data_lo << (8*addr_overflow)) | (data_hi >> 8*(sizeof(u64)-addr_overflow));
	}
	else
	{
		return user_get64(addr);
	}
}


void xorhack_set8(u64 addr, u8 data)
{
	user_set8(addr, data);
}

void xorhack_set16(u64 addr, u16 data)
{
	u64 mask = sizeof(data)-1;
	u64 addr_overflow = addr & mask;
	u64 addr_masked = addr & ~mask;
	
	if(addr_overflow)
	{
		u64 tmp = user_get32(addr_masked);
		tmp = ((u64)data << (8*(sizeof(data)-addr_overflow))) | (tmp & ~(0xFFFFull << (8*(sizeof(data)-addr_overflow))));
		user_set32(addr_masked, tmp);
	}
	else
	{
		user_set16(addr, data);
	}
}

void xorhack_set32(u64 addr, u32 data)
{
	u64 mask = sizeof(data)-1;
	u64 addr_overflow = addr & mask;
	u64 addr_masked = addr & ~mask;
	
	if(addr_overflow)
	{
		u64 tmp = user_get64(addr_masked);
		tmp = ((u64)data << (8*(sizeof(data)-addr_overflow))) | (tmp & ~(0xFFFFFFFFull << (8*(sizeof(data)-addr_overflow))));
		user_set64(addr_masked, tmp);
	}
	else
	{
		user_set32(addr, data);
	}
}

void xorhack_set64(u64 addr, u64 data)
{
	u64 mask = sizeof(u64)-1;
	u64 addr_overflow = addr & mask;
	u64 addr_masked = addr & ~mask;
	
	if(addr_overflow)
	{
		u64 data_lo = user_get64(addr_masked + 0);
		u64 data_hi = user_get64(addr_masked + 8);
		data_lo = (data >> (8*addr_overflow)) | (data_lo & ~(0xFFFFFFFFFFFFFFFFull >> (8*addr_overflow)));
		data_hi = (data << (8*(sizeof(u64)-addr_overflow))) | (data_hi & ~(0xFFFFFFFFFFFFFFFFull << (8*(sizeof(u64)-addr_overflow))));
		user_set64(addr_masked + 0, data_lo);
		user_set64(addr_masked + 8, data_hi);
	}
	else
	{
		user_set64(addr, data);
	}
}

